#import <UIKit/UIKit.h>

#define NCELLS 4

@interface HelloController : UITableViewController
{
	NSMutableArray *tableTitles;
	int ithTitle;
}
@end

@implementation HelloController

- (HelloController *) init
{
	if (!(self = [super init])) return self;
		
	self.title = @"Table Edits";
	
	// Initialize the table titles, so they can be edited over time
	tableTitles = [[NSMutableArray alloc] init];
	ithTitle = NCELLS;
	for (int i = 1; i <= NCELLS; i++) 
		[tableTitles addObject:[NSString stringWithFormat:@"Table Cell #%d", i]];

	return self;
}

#pragma mark UITableViewDataSource Methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView 
{
	return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section 
{
	return [tableTitles count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"any-cell"];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:@"any-cell"] autorelease];
	}

	cell.text = [tableTitles objectAtIndex:[indexPath row]];
	// cell.editingStyle = UITableViewCellEditingStyleDelete; // now read-only and no longer needed
	return cell;
}

- (void) add
{
	[tableTitles addObject:[NSString stringWithFormat:@"Table Cell #%d", ++ithTitle]];
	[self.tableView reloadData];
}

#pragma mark UITableViewDelegateMethods
- (void) deselect
{	
	[self.tableView deselectRowAtIndexPath:[self.tableView indexPathForSelectedRow] animated:YES];
}

// Respond to user selection
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)newIndexPath
{
	printf("User selected row %d\n", [newIndexPath row] + 1);
	[self performSelector:@selector(deselect) withObject:nil afterDelay:0.5f];
}

-(void)leaveEditMode
{
	// Add the edit button
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc]
											  initWithTitle:@"Edit" 
											  style:UIBarButtonItemStylePlain 
											  target:self 
											  action:@selector(enterEditMode)] autorelease];
	[self.tableView endUpdates];
	[self.tableView setEditing:NO animated:YES];
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle 
forRowAtIndexPath:(NSIndexPath *)indexPath 
{
	printf("About to delete item %d\n", [indexPath row]);
	[tableTitles removeObjectAtIndex:[indexPath row]];
	[tableView reloadData];
}	 

-(void)enterEditMode
{
	// Add the edit button
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc]
											  initWithTitle:@"Done" 
											  style:UIBarButtonItemStylePlain 
											  target:self 
											   action:@selector(leaveEditMode)] autorelease];
	[self.tableView setEditing:YES animated:YES];
	[self.tableView beginUpdates];
}

/*
 * Please note that Apple now supplies an edit button that more or less does everything this edit button does,
 * including the EDIT/DONE title swap. This older version does not have the tint-color that highlights the edit function.
 * 
 * If you use 	self.navigationItem.rightBarButtonItem = self.editButtonItem;
 * then you can skip the enter edit mode and leave edit mode functions. You do, however, still need to implement the
 * delegate method(s) (e.g. commitEditingStyle: forRowAtIndexPath:) to make this all work.
 *
 * The reorder sample (next one in this chapter) has been updated to use the new style edits
 */

- (void)loadView
{
	[super loadView];
	
	// Add an add button
	self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc]
											 initWithTitle:@"New" 
											 style:UIBarButtonItemStylePlain 
											 target:self 
											 action:@selector(add)] autorelease];
	
	// Add the edit button
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc]
											 initWithTitle:@"Edit" 
											 style:UIBarButtonItemStylePlain 
											 target:self 
											 action:@selector(enterEditMode)] autorelease];
}

-(void) dealloc
{
	[tableTitles release];
	[super dealloc];
}
@end

@interface SampleAppDelegate : NSObject <UIApplicationDelegate> 
{
	UINavigationController *nav;
}
@property (nonatomic, retain)		UINavigationController *nav;
@end

@implementation SampleAppDelegate
@synthesize nav;
- (void)applicationDidFinishLaunching:(UIApplication *)application {	
	UIWindow *window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
	self.nav = [[UINavigationController alloc] initWithRootViewController:[[HelloController alloc] init]];
	[window addSubview:self.nav.view];
	[window makeKeyAndVisible];
}

- (void) dealloc
{
	[self.nav release];
	[super dealloc];
}
@end

int main(int argc, char *argv[])
{
	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
	int retVal = UIApplicationMain(argc, argv, nil, @"SampleAppDelegate");
	[pool release];
	return retVal;
}
